<?php

/**
 * @file
 * Contains the "searchstax_settings_form" form.
 */

/**
 * Form constructor for the SearchStax settings form.
 *
 * @ingroup forms
 *
 * @see searchstax_settings_form_validate()
 * @see searchstax_settings_form_submit()
 */
function searchstax_settings_form(array $form, array &$form_state): array {
  $config = searchstax_get_utility_service()->getConfig();

  $form['js_version'] = [
    '#type' => 'select',
    '#title' => t('Javascript library version'),
    '#description' => t('The version of the SearchStax Studio tracking library to use for your site. You should usually prefer newer versions, but temporarily using an old version might be necessary in case you have custom additions to the tracking code on your site.'),
    '#options' => [
      '2' => '2',
      '3' => '3',
    ],
    '#default_value' => $config['js_version'],
  ];

  $form['analytics_url'] = [
    '#type' => 'textfield',
    '#title' => t('Analytics URL'),
    '#description' => t("The SearchStudio analytics URL associated with this website. You can find this in your App's settings, under \"Analytics API\". Only available for Javascript library version 3 or higher."),
    '#default_value' => $config['analytics_url'],
    '#states' => [
      'visible' => [
        ':input[name="js_version"]' => [
          'value' => '3',
        ],
      ],
    ],
  ];

  $form['analytics_key'] = [
    '#type' => 'textfield',
    '#title' => t('Global analytics key'),
    '#description' => t('The SearchStudio analytics key associated with this website. You can override this on a per-search basis below, if you have more than one search app configured for this site.'),
    '#default_value' => $config['analytics_key'],
  ];

  $form['search_specific_analytics_keys'] = [
    '#type' => 'details',
    '#title' => t('Search-specific analytics keys'),
    '#description' => t('If you have multiple search apps configured in SearchStax for this website, please enter the analytics key to use for each individual search below. Leave a field blank to use the global key set above for that search – or disable tracking for a search, in case you have left the global key empty, too.'),
    '#tree' => TRUE,
  ];
  foreach (_searchstax_settings_form_get_displays() as $display_id => $display_label) {
    $form['search_specific_analytics_keys'][$display_id] = [
      '#type' => 'textfield',
      '#title' => $display_label,
      '#default_value' => $config["search_specific_analytics_keys.$display_id"] ?? '',
    ];
  }

  try {
    $form['untracked_roles'] = [
      '#type' => 'checkboxes',
      '#title' => t('Do not track these roles'),
      '#description' => t('Select roles for which you want to disable tracking of search behavior.'),
      '#options' => user_roles(),
      '#default_value' => $config['untracked_roles'] ?: [],
    ];
  }
  catch (Exception $e) {
    watchdog_exception('searchstax', $e);
  }

  if (module_exists('search_api_autocomplete')) {
    $form['autosuggest_core'] = [
      '#type' => 'textfield',
      '#title' => t('Auto-suggest core'),
      '#description' => t('The “core” part of your SearchStudio auto-suggest URL. This is the second-to-last part of the URL, between “solr” and “emsuggest”: [DOMAIN]/solr/[CORE]/emsuggest. (Only needed if you want to use auto-suggest.)'),
      '#default_value' => $config['autosuggest_core'],
    ];
  }
  // @todo Check whether /emselect is available before allowing to enable?
  $form['searches_via_searchstudio'] = [
    '#type' => 'checkbox',
    '#title' => t('Re-route searches through SearchStudio'),
    '#description' => t('Re-route all Solr searches through the SearchStudio search handler, to immediately apply adjustments you make in SearchStudio.'),
    '#default_value' => (bool) $config['searches_via_searchstudio'],
  ];
  $form['configure_via_searchstudio'] = [
    '#type' => 'checkbox',
    '#title' => t('Configure searches via SearchStudio'),
    '#description' => t('If this option is enabled, search settings configured in Drupal will be ignored and the settings configured in SearchStudio will take effect. You can configure which settings exactly will be affected. Fulltext keys, filters and paging parameters are always passed to the Solr server and are never affected by this setting.'),
    '#default_value' => (bool) $config['configure_via_searchstudio'],
    '#states' => [
      'visible' => [
        ':input[name="searches_via_searchstudio"]' => ['checked' => TRUE],
      ],
    ],
  ];
  $form['discard_parameters'] = [
    '#type' => 'checkboxes',
    '#title' => t('Which Drupal search settings should be ignored?'),
    '#description' => t('The selected Drupal settings will be ignored in the search request sent to SearchStax. Instead, the settings made within SearchStudio will take effect. (Otherwise, the Drupal settings will override the ones in SearchStudio.) This has no effect when there are no Drupal settings for the specific component – for instance, when spellchecking has not been enabled.'),
    '#options' => [
      'keys' => t('Parse mode and searched fields'),
      'highlight' => t('Highlighting settings'),
      'spellcheck' => t('Spellcheck settings'),
      'sort' => t('Sorts'),
      'facets' => t('Facets'),
    ],
    'facets' => [
      '#description' => t('This will in most cases not work correctly with the Facets module, so custom code handling the returned facets will be needed.'),
    ],
    '#default_value' => $config['discard_parameters'],
    '#states' => [
      'visible' => [
        ':input[name="searches_via_searchstudio"]' => ['checked' => TRUE],
        ':input[name="configure_via_searchstudio"]' => ['checked' => TRUE],
      ],
    ],
  ];

  $form['actions']['#type'] = 'actions';
  $form['actions']['submit'] = [
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  ];

  return $form;
}

/**
 * Retrieves all search displays recognized by this module.
 *
 * @return string[]
 *   An associative array of search displays, indexed by their IDs and with
 *   their human-readable labels as values.
 */
function _searchstax_settings_form_get_displays(): array {
  $displays = [];

  // Views
  if (module_exists('views')) {
    $prefix = 'search_api_index_';
    $prefix_len = strlen($prefix);
    /** @var view $view */
    foreach (views_get_all_views() as $view_id => $view) {
      if (substr($view->base_table, 0, $prefix_len) !== $prefix) {
        continue;
      }
      $view->init_display();
      /** @var views_display[] $view_displays */
      $view_displays = $view->display;
      unset($view_displays['default']);
      foreach ($view_displays as $display_id => $display) {
        $displays["views.$view_id.$display_id"] = t('View %view, display %display', [
          '%view' => $view->human_name ?: $view_id,
          '%display' => $display->display_title ?? $display_id,
        ]);
      }
    }
  }

  // Search API Pages
  if (module_exists('search_api_page')) {
    foreach (search_api_page_load_multiple(FALSE, ['enabled' => TRUE]) as $page) {
      $displays["search_api_page.{$page->machine_name}"] = t('Search page %title', [
        '%title' => $page->name ?: $page->machine_name,
      ]);
    }
  }

  return $displays;
}

/**
 * Form validation handler for searchstax_settings_form().
 *
 * @see searchstax_settings_form()
 */
function searchstax_settings_form_validate(array $form, array &$form_state) {
  if (
    !empty($form['autosuggest_core'])
    && strpos($form_state['values']['autosuggest_core'], '/') !== FALSE
  ) {
    $vars = ['@name' => $form['autosuggest_core']['#title']];
    $message = t('Please enter @name without any slashes (/).', $vars);
    form_set_error('autosuggest_core', $message);
  }
}

/**
* Form submission handler for searchstax_settings_form().
*
* @see searchstax_settings_form()
*/
function searchstax_settings_form_submit(array $form, array &$form_state) {
  $config = searchstax_get_utility_service()->getConfig();
  $config['js_version'] = $form_state['values']['js_version'];
  $config['analytics_url'] = $form_state['values']['analytics_url'];
  $config['analytics_key'] = $form_state['values']['analytics_key'];
  $analytics_keys = array_filter($form_state['values']['search_specific_analytics_keys'] ?: []);
  $config['search_specific_analytics_keys'] = $analytics_keys;
  $untracked_roles = array_keys(array_filter($form_state['values']['untracked_roles']));
  $config['untracked_roles'] = $untracked_roles;
  $config['autosuggest_core'] = $form_state['values']['autosuggest_core'];
  $config['searches_via_searchstudio'] = $form_state['values']['searches_via_searchstudio'];
  $config['configure_via_searchstudio'] = $form_state['values']['configure_via_searchstudio'];
  $discard_parameters = array_keys(array_filter($form_state['values']['discard_parameters']));
  $config['discard_parameters'] = $discard_parameters;
  variable_set('searchstax.settings', $config);

  drupal_set_message(t('The configuration options have been saved.'));
}
