<?php

/**
 * Provides utility methods for the SearchStax module.
 */
class SearchStax implements SearchStaxServiceInterface {

  /**
   * {@inheritdoc}
   */
  public function getConfig(): array {
    $config = variable_get('searchstax.settings', []);
    $config += [
      'analytics_key' => '',
      'analytics_url' => 'https://app.searchstax.com',
      'autosuggest_core' => '',
      'configure_via_searchstudio' => TRUE,
      'discard_parameters' => [
        'highlight',
        'keys',
        'sort',
        'spellcheck',
      ],
      'js_version' => '3',
      'searches_via_searchstudio' => FALSE,
      'untracked_roles' => [],
    ];
    return $config;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfigEntry(string $name) {
    $config = $this->getConfig();
    return $config[$name] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isTrackingDisabled(): bool {
    global $user;

    $roles = $this->getConfigEntry('untracked_roles');
    if (!$roles) {
      return FALSE;
    }
    return (bool) array_intersect_key($user->roles, array_flip($roles));
  }

  /**
   * {@inheritdoc}
   */
  public function addTracking(
    array $result_set,
    SearchApiQueryInterface $query,
    array &$build = [],
    ?string $keys = NULL
  ): bool {
    global $language, $user;

    // Double-check that tracking isn't disabled for the current user.
    if ($this->isTrackingDisabled()) {
      return FALSE;
    }

    // Extract keys if not passed already.
    $keys = $keys ?? $this->getQueryKeys($query);
    if ($keys === NULL) {
      return FALSE;
    }

    // Get the analytics key. Return early if it is not set.
    $config = $this->getConfig();
    $search_id = $query->getOption('search id');
    $config_key = "search_specific_analytics_keys.$search_id";
    $analytics_key = $config[$config_key] ?? $config['analytics_key'];
    if (!$analytics_key) {
      return FALSE;
    }

    try {
      $datasource = $query->getIndex()->datasource();
    }
    catch (SearchApiException $e) {
      watchdog_exception('searchstax', $e);
      return FALSE;
    }
    $offset = $query->getOption('offset', 0);
    /** @var array[] $result_items */
    $result_items = array_values($result_set['results'] ?? []);

    // Pre-load any missing result items to make use of multi-loading.
    $to_load = [];
    foreach ($result_items as $i => $item) {
      if (empty($item['entity'])) {
        $to_load[$i] = $item['id'];
      }
    }
    if ($to_load) {
      try {
        $items = $datasource->loadItems($to_load);
        foreach ($to_load as $i => $id) {
          if (!empty($items[$id])) {
            $result_items[$i]['entity'] = $items[$id];
          }
        }
      }
      catch (SearchApiDataSourceException $e) {
        // Ignore.
      }
    }

    $urls = [];
    foreach ($result_items as $i => $item) {
      /** @var EntityInterface $entity */
      $entity = $item['entity'] ?? NULL;
      if (!$entity) {
        continue;
      }
      try {
        $url = $datasource->getItemUrl($entity);
        if (!$url) {
          continue;
        }
        $urls[] = [
          'url' => url($url['path'], $url['options']),
          'position' => $offset + $i + 1,
        ];
      }
      catch (SearchApiException $ignored) {
        continue;
      }
    }

    $page_no = 1;
    $limit = $query->getOption('limit');
    if ($limit) {
      $page_no += floor($offset / $limit);
    }
    $search_info = [
      'query' => $keys,
      'shownHits' => count($result_items),
      'totalHits' => $result_set['result count'],
      'pageNo' => $page_no,
      'tracked' => FALSE,
      'language' => $language->language ?? 'en',
    ];
    if (!empty($result_set['performance']['execution'])) {
      $search_info['latency'] = (int) ($result_set['performance']['execution'] * 1000);
    }

    foreach ($result_items as $i => $item) {
      $item_info = [
        'cDocId' => $item['id'],
        'position' => $offset + $i + 1,
      ];
      try {
        $object = $item['entity'] ?? NULL;
        if ($object) {
          $item_info['cDocTitle'] = $datasource->getItemLabel($object);
        }
      }
      catch (SearchApiException $ignored) {
        // The item label is not that important.
      }
      $search_info['impressions'][] = $item_info;
    }

    $tracking_info = [
      'key' => $analytics_key,
    ];

    // Add the session ID, if there is a session yet. Otherwise, we don't want
    // to create one, as that would probably end up in the static page cache,
    // which would mean different anonymous visitors would all get the same
    // session ID assigned for that page (but still a new one for each
    // different page). Instead, if no session key is passed via the settings,
    // we manually create a random new one in Javascript.
    // @todo This is not persistent for anonymous users. Is this what we want?
    if (drupal_session_started()) {
      $tracking_info['session'] = session_id();
    }

    // For non-anonymous users, also include the user ID.
    if (!empty($user->uid)) {
      $tracking_info['user'] = $user->uid;
    }

    drupal_add_js(drupal_get_path('module', 'searchstax') . '/js/searchstax.tracking.js');
    $settings = [
      'analytics_url' => rtrim($config['analytics_url'], '/'),
      'js_version' => $config['js_version'],
      'tracking_base_data' => $tracking_info,
      'searches' => [
        $search_id => $search_info,
      ],
    ];
    if ($urls) {
      $settings['results_urls'][$search_id] = $urls;
    }
    drupal_add_js(['searchstax' => $settings], 'setting');

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function alterSolrQuery(array &$call_args, SearchApiQueryInterface $query): void {
    $keys = &$call_args['query'];
    $params = &$call_args['params'];

    try {
      // Check whether this is even a SearchStax server, and not a regular Solr
      // server.
      $solr_config = $query->getIndex()->server()->options;
      if (!$this->isSearchstaxSolr($solr_config)) {
        return;
      }

      // Check whether user enabled re-routing searches to SearchStudio.
      $config = $this->getConfig();
      if (!$config['searches_via_searchstudio']) {
        return;
      }

      $params[SearchStaxSolrConnection::OVERRIDE_HANDLER_PARAM] = 'emselect';

      // Check whether we should also remove most search params set by Drupal
      // and simplify search keys so SearchStudio settings will take effect.
      if (!$config['configure_via_searchstudio']) {
        return;
      }
      $discard_parameters = (array) $config['discard_parameters'];

      if (in_array('keys', $discard_parameters)) {
        // Make sure search keywords are passed as-is, not re-written.
        $keys = $this->getQueryKeys($query);
        unset($params['qf']);
      }

      // Also remove other feature components, sorts and facets, as configured.
      $params_to_remove = [];
      if (in_array('facets', $discard_parameters)) {
        $params_to_remove[] = 'facet';
      }
      if (in_array('highlight', $discard_parameters)) {
        $params_to_remove[] = 'hl';
      }
      if (in_array('sort', $discard_parameters)) {
        unset($params['sort']);
      }
      if (in_array('spellcheck', $discard_parameters)) {
        $params_to_remove[] = 'spellcheck';
      }

      if ($params_to_remove) {
        $callback = fn ($key) => !array_intersect($params_to_remove, explode('.', $key));
        $params = array_filter($params, $callback, ARRAY_FILTER_USE_KEY);
      }
    }
    catch (SearchApiException $ignored) {
      // Very unlikely, but just ignore if it happens – will cause other errors
      // later in the page request.
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isSearchstaxSolr(array $config): bool {
    $host = $config['host'] ?? '';
    $host_parts = explode('.', $host);
    $n = count($host_parts);
    // If host is "localhost" or similar,
    // $host_parts just contains a single item.
    if ($n < 2) {
      return FALSE;
    }
    $host_suffix = $host_parts[$n - 2] . '.' . $host_parts[$n - 1];
    return in_array($host_suffix, ['searchstax.com', 'searchstax.co']);
  }

  /**
   * {@inheritdoc}
   */
  public function getQueryKeys(SearchApiQueryInterface $query): ?string {
    $keys = $query->getOriginalKeys();
    if (is_array($keys)) {
      $keys = $this->stringifyComplexKeys($keys);
    }
    return $keys;
  }

  /**
   * Extracts search keywords from an array.
   *
   * @param array $keys
   *   The keywords, in the array format described in
   *   SearchApiQueryInterface::getKeys().
   *
   * @return string|null
   *   A string representing the keywords. Or NULL if there were no keywords.
   */
  protected function stringifyComplexKeys(array $keys): ?string {
    $extracted = [];
    foreach ($keys as $i => $key) {
      if (!element_child($i)) {
        continue;
      }
      if (is_string($key) && $key !== '') {
        $extracted[] = $key;
      }
      elseif (is_array($key)) {
        $key = $this->stringifyComplexKeys($key);
        if (!$key) {
          continue;
        }
        if (substr($key, 0, 2) !== '-(') {
          $key = "($key)";
        }
        $extracted[] = $key;
      }
    }

    if (!$extracted) {
      return NULL;
    }

    $glue = ($keys['#conjunction'] ?? 'AND') == 'AND' ? ' ' : ' OR ';
    $extracted_keys = implode($glue, $extracted);
    if (!empty($keys['#negation'])) {
      $extracted_keys = "-($extracted_keys)";
    }
    return $extracted_keys;
  }

}
